
/**************************************************************************
 *
 *  $Id: mbgtime.h 1.37 2019/08/28 13:19:40 martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Definitions and prototypes for mbgtime.c.
 *
 * -----------------------------------------------------------------------
 *  $Log: mbgtime.h $
 *  Revision 1.37  2019/08/28 13:19:40  martin
 *  New structures MBG_TZ_INFO and MBG_LS_INFO which
 *  are used by some new functions in new module mbgtimex.c.
 *  Revision 1.36  2019/08/02 07:51:30  martin
 *  New type MBG_TIME64_T.
 *  Revision 1.35  2019/07/23 07:27:37  martin
 *  Added definition GPS_INITIAL_DAY.
 *  Revision 1.34  2019/07/19 08:39:19  martin
 *  Don't mark frac_sec_from_bin() as deprecated anymore.
 *  Revision 1.33  2019/07/19 07:49:22Z  martin
 *  Modified dec_frac_to_bin_frac_{16,32} so that no compiler
 *  warning is emitted if a large data type is passed as an argument.
 *  Revision 1.32  2019/06/26 10:03:02Z  martin
 *  Updated function prototypes.
 *  Revision 1.31  2019/02/06 10:08:09  martin
 *  Added symbol HNS_PER_MIN.
 *  Revision 1.30  2018/11/26 12:04:44Z  martin
 *  Moved definition NTP_FRAC_PER_SEC here.
 *  Revision 1.29  2018/02/28 16:58:10  martin
 *  Removed reference to frac_sec_from_bin().
 *  Revision 1.28  2018/01/15 18:18:49  martin
 *  Renamed symbol NSECS_PER_SEC to NSEC_PER_SEC.
 *  according to namings of similar symbols.
 *  Revision 1.27  2017/11/29 11:14:57  gregoire
 *  Added Multiplier MSEC_TO_NSEC_MULTIPLIER, MSEC_TO_USEC_MULTIPLIER.
 *  Revision 1.26  2017/11/16 13:33:46  philipp
 *  Added USEC_PER_SEC define.
 *  Revision 1.25  2017/08/15 15:48:59  martin
 *  Define NSECS_PER_SEC only if it hasn't been defined before.
 *  Revision 1.24  2017/07/04 14:02:25  martin
 *  Made definitions of some constants signed to avoid
 *  signed/unsigned compiler warnings.
 *  Moved some macros and inline functions for fraction
 *  conversion here. They are excluded from build in kernel
 *  mode, though, since some kernels don't support this properly.
 *  Moved some NANO_TIME- and NANO_TIME_64-related inline
 *  functions to new module nanotime.c.
 *  Renamed PCPS_HRT_BIN_FRAC_SCALE to MBG_FRAC32_UNITS_PER_SEC.
 *  Updated function prototypes.
 *  Doxygen stuff.
 *  Revision 1.23  2017/03/16 12:26:13  martin
 *  Updated function prototypes.
 *  Revision 1.22  2017/01/25 13:10:55  gregoire.diehl
 *  nano_time_64_to_double and double_to_nano_time_64 added
 *  Revision 1.21  2016/12/15 17:44:59Z  martin
 *  Changed conditions to include time.h.
 *  Fixed spelling.
 *  Removed trailing spaces.
 *  Revision 1.20  2014/05/27 08:09:19  martin
 *  Added NTP_SEC_BIAS.
 *  Revision 1.19  2013/05/22 16:47:01  martin
 *  Added some useful macros.
 *  Revision 1.18  2012/10/02 18:51:11  martin
 *  Include <time.h> for WIN32 target and firmware only
 *  Fixed build under QNX, DOS, and FreeBSD.
 *  Revision 1.17  2010/08/06 13:03:03  martin
 *  Removed obsolete code.
 *  Revision 1.16  2010/07/16 10:22:07Z  martin
 *  Moved definitions of HNS_PER_SEC and HNS_PER_MS here.
 *  Conditionally define FILETIME_1970.
 *  Defined MASK_CLOCK_T for ARM/Cortex.
 *  Revision 1.15  2009/10/23 09:55:21  martin
 *  Added MJD numbers for commonly used epochs.
 *  Revision 1.14  2009/08/12 10:28:12  daniel
 *  Added definition NSECS_PER_SEC.
 *  Revision 1.13  2009/06/12 13:31:44Z  martin
 *  Fix build errors with arm-linux-gcc.
 *  Revision 1.12  2009/03/27 14:14:00  martin
 *  Cleanup for CVI.
 *  Revision 1.11  2009/03/13 09:30:06Z  martin
 *  Include mystd.h in mbgtime.c rather than here. The bit type used
 *  here is now defined in words.h.
 *  Updated comments for GPS_SEC_BIAS.
 *  Revision 1.10  2008/12/11 10:45:41Z  martin
 *  Added clock_t mask for gcc (GnuC).
 *  Revision 1.9  2006/08/25 09:33:46Z  martin
 *  Updated function prototypes.
 *  Revision 1.8  2004/12/28 11:29:02Z  martin
 *  Added macro _n_days.
 *  Updated function prototypes.
 *  Revision 1.7  2002/09/06 07:15:48Z  martin
 *  Added MASK_CLOCK_T for Linux.
 *  Revision 1.6  2002/02/25 08:37:44  Andre
 *  definition MASK_CLOCK_T for ARM added
 *  Revision 1.5  2001/03/02 10:18:10Z  MARTIN
 *  Added MASK_CLOCK_T for Watcom C.
 *  Revision 1.4  2000/09/15 07:57:53  MARTIN
 *  Removed outdated function prototypes.
 *  Revision 1.3  2000/07/21 14:05:18  MARTIN
 *  Defined some new constants.
 *
 **************************************************************************/

#ifndef _MBGTIME_H
#define _MBGTIME_H


/* Other headers to be included */

#include <gpsdefs.h>

#if !defined( MBG_TGT_KERNEL ) || defined( MBG_TGT_WIN32 )
  #include <time.h>
#endif


#ifdef __cplusplus
extern "C" {
#endif

#ifdef _MBGTIME
 #define _ext
 #define _DO_INIT
#else
 #define _ext extern
#endif


/* Start of header body */

/**
 * @brief A POSIX-like timestamp which is always 64 bits wide.
 *
 * Depending on the build and target environment, the original
 * POSIX time_t may be only 32 bits wide, and thus will roll over
 * in year 2038. Using this type for time conversion functions
 * which also support this avoid the rollover on such systems.
 */
typedef int64_t MBG_TIME64_T;



/**
 * @brief The number of days from 0000-01-01 until GPS epoch
 *
 * The number of days as computed by ::n_days for the date
 * of the GPS epoch, 1980-01-06.
 *
 * @see ::n_days
 */
#define GPS_INITIAL_DAY  722819L


/**
 * @brief GPS epoch bias from ordinary time_t epoch
 *
 * The Unix time_t epoch is usually 1970-01-01 00:00 whereas
 * the GPS epoch is 1980-01-06 00:00, so the difference is 10 years,
 * plus 2 days due to leap years (1972 and 1976), plus the difference
 * of the day-of-month (6 - 1), so:<br>
 *
 * time_t t = ( gps_week * ::SECS_PER_WEEK ) + sec_of_week + ::GPS_SEC_BIAS
 */
#define GPS_SEC_BIAS   315964800UL     // ( ( ( 10UL * 365UL ) + 2 + 5 ) * SECS_PER_DAY )


/**
 * @brief NTP epoch bias from ordinary time_t epoch
 *
 * The Unix time_t epoch is usually 1970-01-01 00:00 whereas
 * the NTP epoch is 1900-01-01 00:00, so the difference is
 * a constant number of seconds:<br>
 *
 * time_t t = ntp_time - ::NTP_SEC_BIAS
 */
#define NTP_SEC_BIAS   2208988800UL

#if !defined( MBG_TGT_MISSING_64_BIT_TYPES )
  #define NTP_FRAC_PER_SEC         (uint64_t) 4294967296.0
#endif



// Modified Julian Day (MJD) numbers for some commonly used epochs.
// To compute the MJD for a given date just compute the days since epoch
// and add the constant number of days according to the epoch, e.g.:
//   current_unix_mjd = ( time( NULL ) / SECS_PER_DAY ) + MJD_AT_UNIX_EPOCH;
#define MJD_AT_GPS_EPOCH    44244UL    // MJD at 1980-01-06
#define MJD_AT_UNIX_EPOCH   40587UL    // MJD at 1970-01-01
#define MJD_AT_NTP_EPOCH    40587UL    // MJD at 1900-01-01


// The constant below defines the Windows FILETIME number (100 ns intervals
// since 1601-01-01) for 1970-01-01, which is usually the epoch for the time_t
// type used by the standard C library.
#if !defined( FILETIME_1970 )
  // FILETIME represents a 64 bit number, so we need to defined the
  // constant with an appendix depending on the compiler.
  #if MBG_TGT_C99 || defined( __GNUC__ )
    // syntax introduced by C99 standard
    #define FILETIME_1970    0x019db1ded53e8000ULL  // Epoch offset from FILETIME to UNIX
  #elif defined( MBG_TGT_WIN32 )
    // MSC-specific syntax
    #define FILETIME_1970    0x019db1ded53e8000ui64
  #endif
#endif


#if defined( _C166 )
  #if _C166 >= 50
    #define MASK_CLOCK_T 0x7FFFFFFFL
  #else
    #define MASK_CLOCK_T 0x7FFF   /* time.h not shipped with compiler */
  #endif
#endif

#if defined( __WATCOMC__ )
  #define MASK_CLOCK_T 0x7FFFFFFFL
#endif

#if defined( _CVI ) || defined( _CVI_ )
  #define MASK_CLOCK_T 0x7FFFFFFFL
#endif

#if defined( _MSC_VER )
  #define MASK_CLOCK_T 0x7FFFFFFFL
#endif

#if defined( __NETWARE_386__ )
  #define MASK_CLOCK_T 0x7FFFFFFFL
#endif

#if defined( __ARM )
  #define MASK_CLOCK_T 0x7FFFFFFFL
#endif

#if defined( __ARMCC_VERSION )
  #define MASK_CLOCK_T ( ( (ulong) (clock_t) -1 ) >> 1 )
#endif

#if defined( __GNUC__ )
  #if defined( __linux )
    #define MASK_CLOCK_T ( ( (ulong) (clock_t) -1 ) >> 1 )
  #else  // Windows / MinGW
    #define MASK_CLOCK_T 0x7FFFFFFFL
  #endif
#endif


#if !defined( MASK_CLOCK_T )
  #if sizeof( clock_t ) == sizeof( short )
    #define MASK_CLOCK_T 0x7FFF
  #elif sizeof( clock_t ) == sizeof( long )
    #define MASK_CLOCK_T 0x7FFFFFFFL
  #endif
#endif

typedef struct
{
  clock_t start;
  clock_t stop;
  short is_set;

} TIMEOUT;


#define DAYS_PER_WEEK     7

#define SECS_PER_MIN      60
#define MINS_PER_HOUR     60
#define HOURS_PER_DAY     24
#define DAYS_PER_WEEK     7

#define MINS_PER_DAY      ( MINS_PER_HOUR * HOURS_PER_DAY )

#define SECS_PER_HOUR     3600
#define SECS_PER_DAY      86400L
#define SECS_PER_WEEK     604800L

#define SEC100S_PER_SEC   100L
#define SEC100S_PER_MIN   ( SEC100S_PER_SEC * SECS_PER_MIN )
#define SEC100S_PER_HOUR  ( SEC100S_PER_SEC * SECS_PER_HOUR )
#define SEC100S_PER_DAY   ( SEC100S_PER_SEC * SECS_PER_DAY )

#if !defined( MSEC_PER_SEC )
  #define MSEC_PER_SEC    1000L
#endif

#define MSEC_PER_MIN      ( MSEC_PER_SEC * SECS_PER_MIN )
#define MSEC_PER_HOUR     ( MSEC_PER_SEC * SECS_PER_HOUR )
#define MSEC_PER_DAY      ( MSEC_PER_SEC * SECS_PER_DAY )

#if !defined( USEC_PER_SEC )
  #define USEC_PER_SEC    1000000L
#endif

#if !defined( NSEC_PER_SEC )
  #define NSEC_PER_SEC   1000000000L
#endif

#if !defined( HNS_PER_SEC )
  #define HNS_PER_SEC     10000000L
#endif

#if !defined( HNS_PER_MS )
  #define HNS_PER_MS      10000L
#endif

#if !defined( HNS_PER_MIN )
  #define HNS_PER_MIN     ( HNS_PER_SEC * SECS_PER_MIN )
#endif

#define MSEC_TO_NSEC_MULTIPLIER    ( NSEC_PER_SEC / MSEC_PER_SEC )
#define MSEC_TO_USEC_MULTIPLIER    ( USEC_PER_SEC / MSEC_PER_SEC )


/**
 * @brief A table with the days of month
 *
 * First row is for standard years, second row is
 * for leap years.
 *
 * @see DAYS_OF_MONTH_TABLE_INIT
 */
typedef char DAYS_OF_MONTH_TABLE[2][12];


/**
 * @brief An initializer for a ::DAYS_OF_MONTH_TABLE
 */
#define DAYS_OF_MONTH_TABLE_INIT                      \
{                                                     \
  { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }, \
  { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }  \
}




_ext TM_GPS dhms;
_ext TM_GPS datum;


_ext const char *short_time_fmt
#ifdef _DO_INIT
 = "%2i:%02i"
#endif
;

_ext const char *time_fmt
#ifdef _DO_INIT
 = "%2i:%02i:%02i"
#endif
;

_ext const char *long_time_fmt
#ifdef _DO_INIT
 = "%2i:%02i:%02i.%02i"
#endif
;

_ext const char *date_fmt
#ifdef _DO_INIT
 = "%2i.%02i.%04i"
#endif
;

_ext const char *day_date_fmt
#ifdef _DO_INIT
 = "%s, %2i.%02i.%04i"
#endif
;

_ext const char *day_name_eng[]
#ifdef _DO_INIT
 = { "Su", "Mo", "Tu", "We", "Th", "Fr", "Sa" }
#endif
;

_ext const char *day_name_ger[]
#ifdef _DO_INIT
 = { "So", "Mo", "Di", "Mi", "Do", "Fr", "Sa" }
#endif
;

_ext const TM_GPS init_tm
#ifdef _DO_INIT
  = { 1980, 1, 1, 0, 0, 0, 0, 0, 0, 0 }
#endif
;


_ext DAYS_OF_MONTH_TABLE days_of_month
#ifdef _DO_INIT
 = DAYS_OF_MONTH_TABLE_INIT
#endif
;


// simplify call to n_days with structures
#define _n_days( _s ) \
  n_days( (_s)->mday, (_s)->month, (_s)->year )


#define _is_leap_year( _y ) \
  ( ( ( ( (_y) % 4 ) == 0 ) && ( ( (_y) % 100 ) != 0 ) ) || ( ( (_y) % 400 ) == 0 ) )


#define _get_days_of_month( _y, _m ) \
  days_of_month[ _is_leap_year( _y ) ][_m]



#if !defined( MBG_TGT_KERNEL )

/**
 * @brief Data type used for intermediate results on 32 bit multiplications
 *
 * We need 64 bits for intermediate integer results to avoid a range overflow
 * from 32 x 32 bit multiplications. On systems which don't support 64 bit types
 * we use the "double" type as a workaround.
 */
#if defined( MBG_TGT_MISSING_64_BIT_TYPES )
  #define MBG_FRAC32_CONVERSION_TYPE double
#else
  #define MBG_FRAC32_CONVERSION_TYPE int64_t
#endif

/**
 * @brief Constant used to convert e.g. ::PCPS_TIME_STAMP::frac values
 *
 * Max value of ::PCPS_TIME_STAMP::frac + 1, used for scaling
 */
#define MBG_FRAC32_UNITS_PER_SEC  ( (MBG_FRAC32_CONVERSION_TYPE) 4294967296.0  )  // == 0x100000000



/**
 * @brief DST on/off times pre-computed for a given year.
 *
 * Used like a cache to avoid redundant expensive computation
 * of DST switching times for a given year inside an application.
 * Not to be used for data exchange between devices.
 *
 * By default, switching times @a #t_on and @a #t_off are stored
 * as local standard time (i.e. %UTC + @a #offs already applied)
 * because this is most suitable for %UTC to local time conversions.
 *
 * However, for some cases (e.g. with PTP SMPTE) the switching
 * times need to be compared to TAI times, so the function
 * ::mbg_tz_info_to_tai can be used to convert the switching
 * times to TAI. Care must be taken that the TAI switching times
 * also need to be updated whenever a leap second event occurrs.
 *
 * @see ::mbg_set_tz_info_for_year
 * @see ::mbg_set_tz_info_for_utc_time_t
 * @see ::mbg_tz_info_to_tai
 */
typedef struct
{
  MBG_TIME64_T t_on;   ///< 'DST on' time, local standard time (default), or TAI.
  MBG_TIME64_T t_off;  ///< 'DST off' time, local standard time (default), or TAI.
  int offs;            ///< Offset to be added to %UTC to yield local standard time [sec].
  int offs_dl;         ///< Additional offset to be added if daylight saving is in effect [sec].
  int year;            ///< The year number for which @a #t_on and @a #t_off have been computed.
  int auto_flag;       ///< A flag indicating that @a #t_on and @a #t_off were computed by automatic rules.
  int valid;           ///< A flag indicating that the information in this structure has been set up.

} MBG_TZ_INFO;



/**
 * @brief Current %UTC/TAI Offset And Leap Second Information.
 *
 * The stored information can be retrieved e.g. from the
 * ::UTC data set transmitted by the GPS satellites,
 * or from an NTP leap second file.
 *
 * @see ::mbg_set_ls_info_from_gps_utc
 */
typedef struct
{
  /// @brief Time of the nearest leap second, if available, in %UTC time scale.
  /// Should match %UTC midnight at the end of the last day in June or December of a given year.
  MBG_TIME64_T t_ls_utc;

  /// @brief Time of the nearest leap second, if available, in TAI time scale.
  /// Should be ahead of @a #t_ls_utc by a number of leap seconds (~37 s in year 2019)
  /// that have already been inserted in the past.
  MBG_TIME64_T t_ls_tai;

  /// @brief Number of seconds to be inserted into the %UTC time scale at the leap second transition.
  /// This is 0 as long as no leap second announcement is currently available, +1 for a leap
  /// second to be inserted, and -1 for a leap second to be deleted, which has yet never happened.
  int ls_step;

  int offs_gps_utc;  ///< Number of seconds the GPS system time is ahead of %UTC after the leap second transition.

  int offs_tai_utc;  ///< Number of seconds TAI is ahead of %UTC after the leap second transition.

  int valid;         ///< Indicates that the structure has been set up.

} MBG_LS_INFO;



/**
 * @brief Convert a 16 bit binary fraction to a scaled decimal
 *
 * @param[in] bin    The binary fraction
 * @param[in] scale  The scale factor
 *
 * @return The calculated number
 *
 * @see ::dec_frac_to_bin_frac_16
 * @see ::dec_frac_to_bin_frac_32
 * @see ::bin_frac_32_to_dec_frac
 */
static __mbg_inline
uint32_t bin_frac_16_to_dec_frac( uint16_t bin, uint32_t scale )
{
  return (uint32_t) ( (MBG_FRAC32_CONVERSION_TYPE) bin * scale
                      / 0x10000UL );

} // bin_frac_16_to_dec_frac



/**
 * @brief Convert a 32 bit binary fraction to a scaled decimal
 *
 * @param[in] bin    The binary fraction
 * @param[in] scale  The scale factor
 *
 * @return The calculated number
 *
 * @see ::dec_frac_to_bin_frac_32
 * @see ::dec_frac_to_bin_frac_16
 * @see ::bin_frac_16_to_dec_frac
 */
static __mbg_inline
uint32_t bin_frac_32_to_dec_frac( uint32_t bin, uint32_t scale )
{
  return (uint32_t) ( (MBG_FRAC32_CONVERSION_TYPE) bin * scale
                      / MBG_FRAC32_UNITS_PER_SEC );

} // bin_frac_32_to_dec_frac



#if !defined( MBG_TGT_MISSING_64_BIT_TYPES )

// On targets which don't provide 64 bit data types
// MBG_FRAC32_CONVERSION_TYPE is defined as double,
// in which case the ">> 1" operation in the 2 functions
// below yields an "invalid use of floating point" error.
// This could probably be fixed by a different way of
// casting, at least for a partial expression.

static __mbg_inline
uint16_t dec_frac_to_bin_frac_16( MBG_FRAC32_CONVERSION_TYPE dec, uint32_t scale )
{
  return (uint16_t) ( ( ( dec * 0x20000 / scale ) + 1 ) >> 1 );

} // dec_frac_to_bin_frac_16


static __mbg_inline
uint32_t dec_frac_to_bin_frac_32( MBG_FRAC32_CONVERSION_TYPE dec, uint32_t scale )
{
  return (uint32_t) ( ( ( dec * MBG_FRAC32_UNITS_PER_SEC * 2 / scale ) + 1 ) >> 1 );

} // dec_frac_to_bin_frac_32

#endif  // !defined( MBG_TGT_MISSING_64_BIT_TYPES )



#define bin_frac_32_to_msec( _bin ) bin_frac_32_to_dec_frac( (_bin), 1000L )
#define bin_frac_32_to_usec( _bin ) bin_frac_32_to_dec_frac( (_bin), 1000000L )
#define bin_frac_32_to_nsec( _bin ) bin_frac_32_to_dec_frac( (_bin), 1000000000L )
#define bin_frac_16_to_msec( _bin ) bin_frac_16_to_dec_frac( (_bin), 1000L )
#define bin_frac_16_to_usec( _bin ) bin_frac_16_to_dec_frac( (_bin), 1000000L )
#define bin_frac_16_to_nsec( _bin ) bin_frac_16_to_dec_frac( (_bin), 1000000000L )


#define msec_to_bin_frac_32( _msec ) dec_frac_to_bin_frac_32( (_msec), 1000L )
#define usec_to_bin_frac_32( _usec ) dec_frac_to_bin_frac_32( (_usec), 1000000L )
#define nsec_to_bin_frac_32( _nsec ) dec_frac_to_bin_frac_32( (_nsec), 1000000000L )
#define msec_to_bin_frac_16( _msec ) dec_frac_to_bin_frac_16( (_msec), 1000L )
#define usec_to_bin_frac_16( _usec ) dec_frac_to_bin_frac_16( (_usec), 1000000L )
#define nsec_to_bin_frac_16( _nsec ) dec_frac_to_bin_frac_16( (_nsec), 1000000000L )



/**
 * @brief Convert a binary fraction to a scaled decimal
 *
 * Convert a binary fraction (e.g. as in ::PCPS_TIME_STAMP::frac)
 * to a decimal fraction, using a specified scale factor. Depending
 * on the @p scale factor, the result can be milliseconds, microseconds,
 * nanoseconds, or whatever.
 *
 * This function is actually just an alias for ::bin_frac_32_to_dec_frac,
 * but has been introduced much erlier than the latter, and thus is kept
 * for compatibility reasons.
 *
 * @param[in] b      The binary fraction
 * @param[in] scale  The scale factor
 *
 * @return The calculated number
 *
 * @see ::bin_frac_32_to_dec_frac
 */
static __mbg_inline
uint32_t frac_sec_from_bin( uint32_t b, uint32_t scale )
{
  return bin_frac_32_to_dec_frac( b, scale );

}  // frac_sec_from_bin



/**
 * @brief Convert a binary fraction to "double" fractions
 *
 * Convert a binary fraction (e.g. of a second, as in ::PCPS_TIME_STAMP::frac)
 * to a "double" with the units of seconds.<br>
 * E.g. a 0xFFFFFFFF fraction yields 0.9999999999....
 *
 * @note Excluded from build for kernel drivers which usually
 * don't support floating point operations.
 *
 * @param[in] b      The binary fraction
 *
 * @return The calculated fraction
 *
 * @see ::MBG_FRAC32_UNITS_PER_SEC
 */
static __mbg_inline
double dfrac_sec_from_bin( uint32_t b )
{
  return (double) b / (double) MBG_FRAC32_UNITS_PER_SEC;

}  // dfrac_sec_from_bin

#endif  // !defined( MBG_TGT_KERNEL )



/* ----- function prototypes begin ----- */

/* This section was generated automatically */
/* by MAKEHDR, do not remove the comments. */

 /**
 * @brief Set a timeout object to specified interval
 *
 * @param[out]  t         The timeout object
 * @param[in]   clk       The current time, in clock_t ticks
 * @param[in]   interval  The interval until expiration, in clock_t ticks
 */
 void set_timeout( TIMEOUT *t, clock_t clk, clock_t interval ) ;

 /**
 * @brief Stretch a timeout specified in given timeout object
 *
 * @param[in,out]  t         The timeout object
 * @param[in]      interval  The interval until expiration, in clock_t ticks
 */
 void stretch_timeout( TIMEOUT *t, clock_t interval ) ;

 /**
 * @brief Check if a timeout object has expired
 *
 * @param[in]  t    The timeout object
 * @param[in]  clk  The current time, in clock_t ticks
 *
 * @return 1 if timeout expired, else 0
 */
 bit check_timeout( TIMEOUT *t, clock_t clk ) ;

 /**
 * @brief Check if a ::TM_GPS structure contains a valid date and time
 *
 * @param[in]  tm   The date/time structure to be checked
 *
 * @return 0 if date/time is valid, else a negative number indicating
 *         which field was found invalid
 */
 int err_tm( const TM_GPS *tm ) ;

 /**
 * @brief Set the time in a ::TM_GPS structure to 00:00:00.000
 *
 * @param[in,out]  tm   The date/time structure to be set
 *
 * @return Pointer to the ::TM_GPS structure that has been passed
 */
 TM_GPS *clear_time( TM_GPS *tm ) ;

 /**
 * @brief Convert second-of-week to day-of-week and time-of-day
 *
 * @param[in]   wsec  The second-of-week number to be converted.
 *                    Must not be negative.
 * @param[out]  tm    Address of a ::TM_GPS structure which takes
 *                    the computed results. Updates the fields
 *                    ::TM_GPS::hour, ::TM_GPS::min, ::TM_GPS::sec,
 *                    and ::TM_GPS::wday in the range 0..6, with
 *                    0 = Sunday.
 *
 * @return  Pointer to the ::TM_GPS structure that has been passed
 *
 * @see ::tm_to_wsec
 * @see ::day_of_week_sun06
 */
 TM_GPS *wsec_to_tm( long wsec, TM_GPS *tm ) ;

 /**
 * @brief Compute second-of-week from day-of-week and time-of-day
 *
 * @todo Specify input / output ranges
 *
 * @param[in]  tm  Address of a ::TM_GPS structure providing day-of-week and time-of-day
 *
 * @return The computed second-of-week number
 *
 * @see ::wsec_to_tm
 */
 long tm_to_wsec( const TM_GPS *tm ) ;

 /**
 * @brief Check if a specific year is a leap year
 *
 * @param[in]  y  The full year number
 *
 * @return  != 0 if the year is a leap year, else 0
 */
 int is_leap_year( int y ) ;

 /**
 * @brief Compute the day-of-year from a given date
 *
 * @param[in]  day    The day-of-month
 * @param[in]  month  The month
 * @param[in]  year   The full year number
 *
 * @return The computed day-of-year
 */
 int day_of_year( int day, int month, int year ) ;

 /**
 * @brief Compute a date from a given year and day-of-year
 *
 * @param[in]   year     The full year number
 * @param[in]   day_num  Number of days from the beginning of that year, may be negative
 * @param[out]  tm       Address of a ::TM_GPS structure which takes the computed results
 */
 void date_of_year ( int year, int day_num, TM_GPS *tm ) ;

 /**
 * @brief Compute day-of-week for a given date.
 *
 * ATTENTION: The computed day-of-week is in the range 0..6,
 * with 0 = Monday (!).
 *
 * In most cases the function ::day_of_week_sun06 is
 * more suitable for applications.
 *
 * @param[in]  day    The day-of-month, 0..31
 * @param[in]  month  The month, 1..12
 * @param[in]  year   The full year number
 *
 * @return The computed day-of-week, 0..6, 0 = Monday (!)
 *
 * @see ::day_of_week_sun06
 * @see ::n_days
 */
 int day_of_week( int day, int month, int year ) ;

 /**
 * @brief Compute day-of-week for a given date.
 *
 * The computed day-of-week is in the range 0..6,
 * with 0 = Sunday, as expected by most applications.
 *
 * @param[in]  day    The day-of-month, 0..31
 * @param[in]  month  The month, 1..12
 * @param[in]  year   The full year number
 *
 * @return The computed day-of-week, 0..6, with 0 = Sunday.
 *
 * @see ::n_days
 * @see ::wsec_to_tm
 */
 int day_of_week_sun06( int day, int month, int year ) ;

 /**
 * @brief Update a year number by a number of days, accounting for leap years
 *
 * @param[in]  day_num  The number of days to evaluate
 * @param[in]  year     The year number to start with
 *
 * @return The computed year number
 */
 int days_to_years( long *day_num, int year ) ;

 /**
 * @brief Compute number of days after Jan 1, 0000 for a given date
 *
 * @param[in]  mday   The day-of-month
 * @param[in]  month  The month
 * @param[in]  year   The full year number
 *
 * @return The computed number of days
 *
 * @see ::day_of_week
 */
 long n_days( ushort mday, ushort month, ushort year ) ;

 /**
 * @brief Print time with hours, minutes, seconds to a string
 *
 * @param[out]  s   Address of a string buffer to be filled
 * @param[in]   tm  Address of a ::TM_GPS structure providing date and time
 */
 int sprint_time( char *s, const TM_GPS *tm ) ;

 /**
 * @brief Print time with hours, minutes to a string
 *
 * @param[out]  s   Address of a string buffer to be filled
 * @param[in]   tm  Address of a ::TM_GPS structure providing date and time
 */
 int sprint_short_time( char *s, const TM_GPS *tm ) ;

 /**
 * @brief Print date to a string
 *
 * @param[out]  s   Address of a string buffer to be filled
 * @param[in]   tm  Address of a ::TM_GPS structure providing date and time
 */
 int sprint_date( char *s, const TM_GPS *tm ) ;

 /**
 * @brief Print day-of-week and date to a string
 *
 * @param[out]  s   Address of a string buffer to be filled
 * @param[in]   tm  Address of a ::TM_GPS structure providing date and time
 */
 int sprint_day_date( char *s, const TM_GPS *tm ) ;

 /**
 * @brief Print day-of-week, date and time to a string
 *
 * @param[out]  s   Address of a string buffer to be filled
 * @param[in]   tm  Address of a ::TM_GPS structure providing date and time
 */
 int sprint_tm( char *s, const TM_GPS *tm ) ;

 /**
 * @brief Extract a time from a string
 *
 * @param[in]   s   A time string in format hh:mm:ss
 * @param[out]  tm  Address of a ::TM_GPS structure which takes the extracted time
 */
 void sscan_time( const char *s, TM_GPS *tm ) ;

 /**
 * @brief Extract a date from a string
 *
 * @param[in]   s   A date string in format dd.mm. or dd.mm.yyyy
 * @param[out]  tm  Address of a ::TM_GPS structure which takes the extracted date
 */
 void sscan_date( char *s, TM_GPS *tm ) ;


/* ----- function prototypes end ----- */


/* End of header body */


#undef _ext
#undef _DO_INIT

#ifdef __cplusplus
}
#endif


#endif  /* _MBGTIME_H */
